package org.vivecraft.gameplay.trackers;

import java.util.HashSet;
import net.minecraft.client.Minecraft;
import net.minecraft.client.multiplayer.ClientLevel;
import net.minecraft.client.player.LocalPlayer;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.core.Registry;
import net.minecraft.network.protocol.game.ServerboundPlayerActionPacket;
import net.minecraft.util.Mth;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.entity.projectile.ProjectileUtil;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.Items;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.state.BlockBehaviour;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.phys.AABB;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.world.phys.EntityHitResult;
import net.minecraft.world.phys.Vec3;
import org.vivecraft.api.VRData;
import org.vivecraft.provider.ControllerType;
import org.vivecraft.reflection.MCReflection;
import org.vivecraft.render.RenderPass;
import org.vivecraft.render.VRFirstPersonArmSwing;
import org.vivecraft.settings.VRHotkeys;
import org.vivecraft.settings.VRSettings;

public class InteractTracker extends Tracker
{
    public boolean[] bukkit = new boolean[2];
    public int hotbar = -1;
    public BlockHitResult[] inBlockHit = new BlockHitResult[2];
    BlockPos[] inBlockPos = new BlockPos[2];
    Entity[] inEntity = new Entity[2];
    private EntityHitResult[] inEntityHit = new EntityHitResult[2];
    private boolean[] inCamera = new boolean[2];
    private boolean[] inHandheldCamera = new boolean[2];
    boolean[] active = new boolean[2];
    boolean[] wasactive = new boolean[2];
    private HashSet<Class> rightClickable = null;

    public InteractTracker(Minecraft mc)
    {
        super(mc);
    }

    public boolean isActive(LocalPlayer p)
    {
        if (this.mc.gameMode == null)
        {
            return false;
        }
        else if (p == null)
        {
            return false;
        }
        else if (!p.isAlive())
        {
            return false;
        }
        else if (p.isSleeping())
        {
            return false;
        }
        else
        {
            Minecraft minecraft = Minecraft.getInstance();

            if (minecraft.vrSettings.seated)
            {
                return false;
            }
            else if (p.isBlocking() && this.hotbar < 0)
            {
                return false;
            }
            else
            {
                return !minecraft.bowTracker.isNotched();
            }
        }
    }

    public void reset(LocalPlayer player)
    {
        for (int i = 0; i < 2; ++i)
        {
            this.reset(player, i);
        }
    }

    private void reset(LocalPlayer player, int c)
    {
        if (this.inCamera[c] && VRHotkeys.isMovingThirdPersonCam() && VRHotkeys.getMovingThirdPersonCamTriggerer() == VRHotkeys.Triggerer.INTERACTION && VRHotkeys.getMovingThirdPersonCamController() == c)
        {
            VRHotkeys.stopMovingThirdPersonCam();
        }

        if (this.inHandheldCamera[c] && this.mc.cameraTracker.isMoving() && this.mc.cameraTracker.getMovingController() == c && !this.mc.cameraTracker.isQuickMode())
        {
            this.mc.cameraTracker.stopMoving();
        }

        this.inBlockPos[c] = null;
        this.inBlockHit[c] = null;
        this.inEntity[c] = null;
        this.inEntityHit[c] = null;
        this.inCamera[c] = false;
        this.inHandheldCamera[c] = false;
        this.active[c] = false;
        this.mc.vr.getInputAction(this.mc.vr.keyVRInteract).setEnabled(ControllerType.values()[c], false);
    }

    public void doProcess(LocalPlayer player)
    {
        if (this.rightClickable == null)
        {
            this.rightClickable = new HashSet<>();

            for (Object object : Registry.BLOCK)
            {
                Class oclass = object.getClass();

                try
                {
                    new MCReflection.ReflectionMethod(oclass, MCReflection.BlockBehavior_Use, BlockState.class, Level.class, BlockPos.class, Player.class, InteractionHand.class, BlockHitResult.class);
                    this.rightClickable.add(oclass);
                }
                catch (Throwable throwable1)
                {
                }

                oclass = oclass.getSuperclass();

                try
                {
                    new MCReflection.ReflectionMethod(oclass, MCReflection.BlockBehavior_Use, BlockState.class, Level.class, BlockPos.class, Player.class, InteractionHand.class, BlockHitResult.class);
                    this.rightClickable.add(oclass);
                }
                catch (Throwable throwable)
                {
                }
            }

            this.rightClickable.remove(Block.class);
            this.rightClickable.remove(BlockBehaviour.class);
            this.rightClickable.remove(BlockBehaviour.BlockStateBase.class);
        }

        Vec3 vec34 = new Vec3(0.0D, 0.0D, -1.0D);

        for (int j = 0; j < 2; ++j)
        {
            if (!this.inCamera[j] && !this.inHandheldCamera[j] || !this.mc.vr.keyVRInteract.isDown(ControllerType.values()[j]))
            {
                this.reset(player, j);

                if (j == 0 && this.hotbar >= 0)
                {
                    this.active[j] = true;
                }

                Vec3 vec35 = this.mc.vrPlayer.vrdata_world_pre.getHeadPivot();
                Vec3 vec3 = this.mc.vrPlayer.vrdata_world_pre.getController(j).getPosition();
                Vec3 vec31 = this.mc.vrPlayer.vrdata_world_pre.getHand(j).getCustomVector(vec34);
                ItemStack itemstack = player.getItemInHand(j == 0 ? InteractionHand.MAIN_HAND : InteractionHand.OFF_HAND);
                Item item = null;

                if (!this.active[j] && (this.mc.vrSettings.displayMirrorMode == VRSettings.MirrorMode.MIXED_REALITY || this.mc.vrSettings.displayMirrorMode == VRSettings.MirrorMode.THIRD_PERSON) && this.mc.vrSettings.mixedRealityRenderCameraModel)
                {
                    VRData.VRDevicePose vrdata$vrdevicepose = this.mc.vrPlayer.vrdata_world_pre.getEye(RenderPass.THIRD);
                    Vec3 vec32 = vrdata$vrdevicepose.getPosition();
                    vec32 = vec32.subtract(vrdata$vrdevicepose.getCustomVector(new Vec3(0.0D, 0.0D, -1.0D)).scale((double)0.15F));
                    vec32 = vec32.subtract(vrdata$vrdevicepose.getCustomVector(new Vec3(0.0D, -1.0D, 0.0D)).scale((double)0.05F));

                    if (vec3.distanceTo(vec32) < (double)0.15F)
                    {
                        this.inCamera[j] = true;
                        this.active[j] = true;
                    }
                }

                if (!this.active[j] && this.mc.cameraTracker.isVisible() && !this.mc.cameraTracker.isQuickMode())
                {
                    VRData.VRDevicePose vrdata$vrdevicepose1 = this.mc.vrPlayer.vrdata_world_pre.getEye(RenderPass.CAMERA);
                    Vec3 vec36 = vrdata$vrdevicepose1.getPosition();
                    vec36 = vec36.subtract(vrdata$vrdevicepose1.getCustomVector(new Vec3(0.0D, 0.0D, -1.0D)).scale((double)0.08F));

                    if (vec3.distanceTo(vec36) < (double)0.11F)
                    {
                        this.inHandheldCamera[j] = true;
                        this.active[j] = true;
                    }
                }

                if (!this.active[j])
                {
                    int k = Mth.floor(vec3.x);
                    int l = Mth.floor(vec3.y);
                    int i = Mth.floor(vec3.z);
                    Vec3 vec33 = new Vec3(vec3.x + vec31.x * -0.1D, vec3.y + vec31.y * -0.1D, vec3.z + vec31.z * -0.1D);
                    AABB aabb = new AABB(vec3, vec33);
                    this.inEntityHit[j] = ProjectileUtil.getEntityHitResult(this.mc.getCameraEntity(), vec35, vec3, aabb, (e) ->
                    {
                        return !e.isSpectator() && e.isPickable() && e != this.mc.getCameraEntity().getVehicle();
                    }, 0.0D);

                    if (this.inEntityHit[j] != null)
                    {
                        Entity entity = this.inEntityHit[j].getEntity();
                        this.inEntity[j] = entity;
                        this.active[j] = true;
                    }
                }

                if (!this.active[j])
                {
                    BlockPos blockpos = null;
                    blockpos = new BlockPos(vec3);
                    BlockState blockstate = this.mc.level.getBlockState(blockpos);
                    BlockHitResult blockhitresult = blockstate.getOcclusionShape(this.mc.level, blockpos).clip(vec35, vec3, blockpos);
                    this.inBlockPos[j] = blockpos;
                    this.inBlockHit[j] = blockhitresult;
                    this.active[j] = blockhitresult != null && (this.rightClickable.contains(blockstate.getBlock().getClass()) || this.rightClickable.contains(blockstate.getBlock().getClass().getSuperclass()));
                    this.bukkit[j] = false;

                    if (!this.active[j] && itemstack.getItem() == Items.BUCKET && blockstate.getMaterial().isLiquid())
                    {
                        this.active[j] = true;
                        this.bukkit[j] = true;
                    }
                }

                if (!this.wasactive[j] && this.active[j])
                {
                    this.mc.vr.triggerHapticPulse(j, 250);
                }

                this.mc.vr.getInputAction(this.mc.vr.keyVRInteract).setEnabled(ControllerType.values()[j], this.active[j]);
                this.wasactive[j] = this.active[j];
            }
        }
    }

    public boolean isInteractActive(int controller)
    {
        return this.active[controller];
    }

    public boolean isInCamera()
    {
        return this.inCamera[0] || this.inCamera[1];
    }

    public boolean isInHandheldCamera()
    {
        return this.inHandheldCamera[0] || this.inHandheldCamera[1];
    }

    public void processBindings()
    {
        for (int i = 0; i < 2; ++i)
        {
            if (this.mc.vr.keyVRInteract.consumeClick(ControllerType.values()[i]) && this.active[i])
            {
                InteractionHand interactionhand = InteractionHand.values()[i];
                boolean flag = false;

                if (this.hotbar >= 0 && this.hotbar < 9 && this.mc.player.getInventory().selected != this.hotbar && interactionhand == InteractionHand.MAIN_HAND)
                {
                    this.mc.player.getInventory().selected = this.hotbar;
                    flag = true;
                }
                else if (this.hotbar == 9 && interactionhand == InteractionHand.MAIN_HAND)
                {
                    this.mc.player.connection.send(new ServerboundPlayerActionPacket(ServerboundPlayerActionPacket.Action.SWAP_ITEM_WITH_OFFHAND, BlockPos.ZERO, Direction.DOWN));
                    flag = true;
                }
                else if (this.inCamera[i])
                {
                    VRHotkeys.startMovingThirdPersonCam(i, VRHotkeys.Triggerer.INTERACTION);
                    flag = true;
                }
                else if (this.inHandheldCamera[i])
                {
                    this.mc.cameraTracker.startMoving(i);
                    flag = true;
                }
                else if (this.inEntityHit[i] != null)
                {
                    flag = true;

                    if (!this.mc.gameMode.interactAt(this.mc.player, this.inEntity[i], this.inEntityHit[i], interactionhand).consumesAction() && !this.mc.gameMode.interact(this.mc.player, this.inEntity[i], interactionhand).consumesAction())
                    {
                        flag = false;
                    }
                }
                else if (this.inBlockHit[i] != null)
                {
                    flag = this.mc.gameMode.useItemOn(this.mc.player, (ClientLevel)this.mc.player.level, interactionhand, this.inBlockHit[i]).consumesAction();
                }
                else if (this.bukkit[i])
                {
                    flag = this.mc.gameMode.useItem(this.mc.player, (ClientLevel)this.mc.player.level, interactionhand).consumesAction();
                }

                if (flag)
                {
                    this.mc.player.swingArm(interactionhand, VRFirstPersonArmSwing.Interact);
                    this.mc.vr.triggerHapticPulse(i, 750);
                }
            }
        }
    }
}
